@@ -1,10 +1,22 @@
+"""Umbrella package for all rules.
+
+Anyone who wants to develop a plugin is welcome to modify the values in this specific namespace. In particular, to add
+a rule with your plugin:
+1) Add your plugin's rules as a submodule of this using
+   `from sys import modules; modules['.'.join((PATH_TO_RULE_MODULE, PATH_TO_YOUR_RULE))] = module[PATH_TO_YOUR_RULE]
+2) Append your plugin's namespace to `rule.__all__`
+3) Append each of your rules' import paths to `consts.AVAILABLE_RULES`
+"""
+
 from __future__ import annotations
 
-from dataclasses import dataclass
 from importlib import import_module
 from typing import Any, Type, cast
 
-from .. import AnyResolution, Rule
+from attrs import define
+
+from .. import Rule
+from ..consts import AnyResolution
 from ..util import dynamic_import
 
 
@@ -19,12 +31,12 @@ def get_rule(type_: str) -> Type[Rule[Any]]:
     raise NameError()
 
 
-@dataclass  # type: ignore
+@define(slots=False)  # type: ignore
 class DoResolveRule(Rule[bool]):
     """The subtype of rule which determines if a market should resolve, returning a bool."""
 
 
-@dataclass  # type: ignore
+@define(slots=False)  # type: ignore
 class ResolutionValueRule(Rule[AnyResolution]):
     """The subtype of rule which determines what a market should resolve to."""
 
