@@ -9,16 +9,19 @@ from typing import TYPE_CHECKING, Dict, Generic, Mapping, Optional, Tuple, Union
 from attrs import Factory, define
 
 from .. import Rule
+from ..caching import parallel
 from ..consts import BinaryResolution, PseudoNumericResolution, T
 from ..util import normalize_mapping
 from . import DoResolveRule, ResolutionValueRule, get_rule
 from .abstract import BinaryRule, ResolveRandomSeed, UnaryRule, VariadicRule
 
 if TYPE_CHECKING:  # pragma: no cover
+    from concurrent.futures import Future
     from typing import Any, ClassVar, DefaultDict, Literal, MutableSequence
 
     from ..consts import FreeResponseResolution, MultipleChoiceResolution
     from ..market import Market
+    from ..util import ModJSONDict
 
 
 @define(slots=False)
@@ -103,6 +106,22 @@ class ImpliesRule(BinaryRule[Optional[BinaryResolution]]):
         return not self.rule1._value(market) or bool(self.rule2._value(market))
 
 
+@define(slots=False)
+class ConditionalRule(BinaryRule[BinaryResolution]):
+    """Cancels if the premise is false, and resolve to another value otherwise."""
+
+    _explainer_stub: ClassVar[str] = (
+        "Cancels if the next line resolves False, otherwise resolves to the value of the item after"
+    )
+
+    def _value(self, market: Market) -> BinaryResolution:
+        f_val1 = parallel(self.rule1._value, market)
+        f_val2 = parallel(self.rule2._value, market)
+        if not f_val1.result():
+            return "CANCEL"
+        return f_val2.result()
+
+
 @define(slots=False)
 class ResolveAtTime(DoResolveRule):
     """Return True if the specified time is in the past."""
@@ -155,10 +174,11 @@ class AdditiveRule(VariadicRule[PseudoNumericResolution]):
     def _value(self, market: Market) -> Literal["CANCEL"] | float:
         """Return the sum of the underlying rules."""
         ret: float = 0
-        for rule in self.rules:
+        futures = [parallel(rule.value, market, format='PSEUDO_NUMERIC') for rule in self.rules]
+        for f_rule in futures:
             val = cast(
                 PseudoNumericResolution,
-                rule.value(market, format='PSEUDO_NUMERIC')
+                f_rule.result()
             )
             if val == "CANCEL":
                 return "CANCEL"
@@ -174,11 +194,12 @@ class MultiplicitiveRule(VariadicRule[PseudoNumericResolution]):
 
     def _value(self, market: Market) -> Literal["CANCEL"] | float:
         """Return the product of the underlying rules."""
-        ret: float = 0
-        for rule in self.rules:
+        ret: float = 1
+        futures = [parallel(rule.value, market, format='PSEUDO_NUMERIC') for rule in self.rules]
+        for f_rule in futures:
             val = cast(
                 PseudoNumericResolution,
-                rule.value(market, format='PSEUDO_NUMERIC')
+                f_rule.result()
             )
             if val == "CANCEL":
                 return "CANCEL"
@@ -222,7 +243,7 @@ class ResolveRandomIndex(ResolveRandomSeed):
 
     def _explain_abstract(self, indent: int = 0, **kwargs: Any) -> str:
         ret = f"{'  ' * indent}- Resolve to a random index, given some original seed. This one operates on a "
-        if self.method == 'randrange':
+        if self.method == 'rand"range':
             ret += f"fixed range of integers in ({self.start} <= x < {self.size}).\n"
         else:
             ret += f"dynamic range based on the current pool and probabilities, but starting at {self.start}.\n"
@@ -238,9 +259,9 @@ class ResolveMultipleValues(ResolutionValueRule):
     def _value(self, market: Market) -> FreeResponseResolution | MultipleChoiceResolution:
         ret: DefaultDict[int, float] = defaultdict(float)
         for rule, part in self.shares:
-            val = cast(Dict[Union[str, int], float], rule.value(market, format='FREE_RESPONSE'))
+            val = cast(Dict[Union[str, int], Future[float]], parallel(rule.value, market, format='FREE_RESPONSE'))
             for idx, value in val.items():
-                ret[int(idx)] += value * part
+                ret[int(idx)] += value.result() * part
         return normalize_mapping(ret)
 
     def _explain_abstract(self, indent: int = 0, **kwargs: Any) -> str:
@@ -252,17 +273,19 @@ class ResolveMultipleValues(ResolutionValueRule):
         return ret
 
     @classmethod
-    def from_dict(cls, env: Mapping[str, Any]) -> 'ResolveMultipleValues':
+    def from_dict(cls, env: ModJSONDict) -> 'ResolveMultipleValues':
         """Take a dictionary and return an instance of the associated class."""
-        env_copy: dict[str, Any] = dict(env)
-        shares: MutableSequence[tuple[ResolutionValueRule | tuple[str, dict[str, Any]], float]] = env['shares']
+        env_copy: ModJSONDict = {**env}
+        shares: MutableSequence[tuple[ResolutionValueRule | tuple[str, ModJSONDict], float]] = (
+            env.get('shares', [])  # type: ignore[assignment]
+        )
         new_shares = []
         for rule, weight in shares:
             try:
-                type_, kwargs = cast(Tuple[str, Dict[str, Any]], rule)
+                type_, kwargs = cast(Tuple[str, ModJSONDict], rule)
                 new_rule = get_rule(type_).from_dict(kwargs)
                 new_shares.append((new_rule, weight))
             except Exception:
                 pass
-        env_copy['shares'] = new_shares
+        env_copy['shares'] = new_shares  # type: ignore
         return super().from_dict(env_copy)
